// Algorithm DDM-2
Function{
  coef_sigma[] = -(1 + I[]*beta/kap*Cos[theta/2]/Cos[theta]);
  coef_gamma[] = beta + I[]*kap*Cos[theta/2];
  If(ABC_ORDER==0)
    ListOfFields_ddm2() = {ListOfFields_g()};
  Else
    ListOfFields_ddm2() = {ListOfFields_g(),ListOfFields_h()};
  EndIf
}

FunctionSpace{
  For ii In {0:#myD()-1}
    i = myD(ii);
    For kk In {0:#ListGammaExt~{i}()-1}
      k=ListGammaExt~{i}(kk);
      {Name H_phi~{i}~{k}; Type Form0;
        BasisFunction{
          {Name phi~{i}~{k}; NameOfCoef phiik; Function BF_Node;
            Support Region[{GammaExt~{i}~{k}, dGammaExt~{i}~{k}}];
            Entity NodesOf[All];}
        }
      }
      // Quantities transfered at each Apt~{i}~{j}~{k}~{l}
      For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
        j=ListdGammaExt~{i}~{k}(index);
        l=ListdGammaExt~{i}~{k}(index+1);
        {Name H_h~{i}~{j}~{k}~{l}; Type Form0;
          BasisFunction{
            {Name h~{i}~{j}~{k}~{l}; NameOfCoef hj; Function BF_Node; 
            Support Region[{Apt~{i}~{j}~{k}~{l}}]; Entity NodesOf[All];}
          }
        }
      EndFor
    EndFor
  EndFor
}

Formulation{
  For ii In {0:#myD()-1}
    i = myD(ii);
    { Name DDM_2~{i}; Type FemEquation;
      Quantity{
      	{Name u~{i} ; Type Local; NameOfSpace H_grad~{i};}
        For kk In {0:#ListGammaExt~{i}()-1}
          k=ListGammaExt~{i}(kk);
          { Name phi~{i}~{k} ; Type Local; NameOfSpace H_phi~{i}~{k};}
        EndFor
      }
      Equation{
        // PDE on u_i
				//Helmholtz equation
        Galerkin{[Dof{Grad u~{i}}, {Grad u~{i}}];
          In Omega~{i}; Jacobian JVol; Integration I1;}
        Galerkin{[-kap^2*Dof{u~{i}}, {u~{i}}];
          In Omega~{i}; Jacobian JVol; Integration I1;}
        // dni ui -i*kap ui = gij on Sigma ij
        For jj In {0:#myD~{i}()-1}
          j = myD~{i}(jj);
          Galerkin{[-I[]*kap*Dof{u~{i}}, {u~{i}}];
            In Sigma~{i}~{j}; Jacobian JSur; Integration I1;}
          Galerkin{[$TCSource?-g~{i}~{j}[]:0., {u~{i}}];
            In Sigma~{i}~{j}; Jacobian JSur; Integration I1;}
        EndFor
        If(ABC_ORDER==0)
					Galerkin{[-I[]*kap*Dof{u~{i}}, {u~{i}}];
						In dOmegaExt~{i}; Jacobian JSur; Integration I1;}
				Else
          // dn u_i = i*omega*phi_k on Gamma_i^k
          For kk In {0:#ListGammaExt~{i}()-1}
            k=ListGammaExt~{i}(kk);
            Galerkin{[-I[]*kap*Dof{phi~{i}~{k}}, {u~{i}}];
              In GammaExt~{i}~{k}; Jacobian JSur; Integration I1;}
          EndFor
          // Operator T (Mandatory : Order = 2 + Corner Correction)
          For kk In {0:#ListGammaExt~{i}()-1}
            k=ListGammaExt~{i}(kk);
            Galerkin{[Dof{phi~{i}~{k}}, {phi~{i}~{k}}];
              In GammaExt~{i}~{k}; Jacobian JSur; Integration I1;}
            Galerkin{[-Dof{u~{i}}, {phi~{i}~{k}}];
              In GammaExt~{i}~{k}; Jacobian JSur; Integration I1;}
            Galerkin{[1/(2*kap^2)*Dof{d phi~{i}~{k}}, {d phi~{i}~{k}}];
              In GammaExt~{i}~{k}; Jacobian JSur; Integration I1;}
            If(CORNER_CONDITION == 2)
              For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
                j=ListdGammaExt~{i}~{k}(index);
                l=ListdGammaExt~{i}~{k}(index+1);
                If (k!=l) // corner point
                  Galerkin{[-Conj[coef_gamma[]]/2/kap/kap/coef_sigma[]*Dof{phi~{i}~{k}}, {phi~{i}~{k}}];
                    In Region[{Apt~{i}~{j}~{k}~{l}}]; Jacobian JLin; Integration I1;}
                  Galerkin{[$TCSource?-h~{i}~{j}~{k}~{l}[]/2/kap/kap:0., {phi~{i}~{k}}];
                    In Region[{Apt~{i}~{j}~{k}~{l}}]; Jacobian JLin; Integration I1;}
                //TODO: Flatpoint 
                //Else 
                EndIf
              EndFor
            EndIf
          EndFor
        EndIf
      }
    }
  EndFor

  //Resolution h
  For ii In {0:#myD()-1}
    i = myD(ii);
    For kk In {0:#ListGammaExt~{i}()-1}
      k=ListGammaExt~{i}(kk);
      For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
        j=ListdGammaExt~{i}~{k}(index);
        l=ListdGammaExt~{i}~{k}(index+1);
        { Name H~{j}~{i}~{l}~{k}; Type FemEquation;
        	Quantity{
        	  { Name phi~{i}~{k} ; Type Local; NameOfSpace H_phi~{i}~{k};}
        	  { Name h~{j}~{i}~{l}~{k} ; Type Local; NameOfSpace H_h~{j}~{i}~{l}~{k};}
    	    }
      	  Equation{
      	    Galerkin{[Dof{h~{j}~{i}~{l}~{k}},{h~{j}~{i}~{l}~{k}}];
      	      In Apt~{i}~{j}~{k}~{l}; Jacobian JLin; Integration I1;}
    	      Galerkin{[$TCSource?Conj[coef_sigma[]]/coef_sigma[]*h~{i}~{j}~{k}~{l}[]:0.,{h~{j}~{i}~{l}~{k}}];
    	        In Apt~{i}~{j}~{k}~{l}; Jacobian JLin; Integration I1;}
    	      Galerkin{[(Conj[coef_sigma[]*coef_gamma[]]/coef_sigma[]/coef_sigma[] + coef_gamma[]/coef_sigma[]) *{phi~{i}~{k}}, {h~{j}~{i}~{l}~{k}}];
    	        In Apt~{i}~{j}~{k}~{l}; Jacobian JLin; Integration I1;}
          }
        }
      EndFor
    EndFor
  EndFor
}

Resolution{
  {Name DDM_2;
    System{
      For ii In {0:#myD()-1}
      	i = myD(ii);
	      {Name DDM_2~{i}; NameOfFormulation DDM_2~{i}; Type Complex; }
        For jj In {0:#myD~{i}()-1}
          j = myD~{i}(jj);
          {Name G~{j}~{i}; NameOfFormulation G~{j}~{i}; Type Complex; }
        EndFor
        If(ABC_ORDER != 0)
          For kk In {0:#ListGammaExt~{i}()-1}
            k=ListGammaExt~{i}(kk);
            For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
              j=ListdGammaExt~{i}~{k}(index);
              l=ListdGammaExt~{i}~{k}(index+1);
              {Name H~{j}~{i}~{l}~{k}; NameOfFormulation H~{j}~{i}~{l}~{k}; Type Complex; }
            EndFor
          EndFor
        EndIf
      EndFor
    }
    Operation{
      // Initialisation
      Evaluate[$PhysicalSource = 1];
      Evaluate[$TCSource = 0];
      For ii In {0:#myD()-1}
        i = myD(ii);
        UpdateConstraint[DDM_2~{i}, dOmegaInt~{i}, Assign];
        Generate[DDM_2~{i}];
        Solve[DDM_2~{i}];
        For jj In {0:#myD~{i}()-1}
          j = myD~{i}(jj);
          Generate[G~{j}~{i}];
          Solve[G~{j}~{i}];
          PostOperation[G~{j}~{i}];
        EndFor
        If(ABC_ORDER != 0)
          For kk In {0:#ListGammaExt~{i}()-1}
            k=ListGammaExt~{i}(kk);
            For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
              j=ListdGammaExt~{i}~{k}(index);
              l=ListdGammaExt~{i}~{k}(index+1);    
              Generate[H~{j}~{i}~{l}~{k}];
              Solve[H~{j}~{i}~{l}~{k}];
              PostOperation[H~{j}~{i}~{l}~{k}];
            EndFor
          EndFor
        EndIf
      EndFor
      // Set uinc=0 and unlock quantity on Sigma
      Evaluate[$PhysicalSource = 0];
      Evaluate[$TCSource = 1];
      For ii In {0:#myD()-1}
      	i = myD(ii);
      	UpdateConstraint[DDM_2~{i}, dOmegaInt~{i}, Assign];
      EndFor

      IterativeLinearSolver["I-A",SOLVER, TOL, MAXIT, RESTART, ListOfFields_ddm2(), {}, {}]
      {
        For ii In {0:#myD()-1}
          i = myD(ii);
          GenerateRHSGroup[DDM_2~{i}, Region[{Sigma~{i}, TrdOmegaInt~{i}, dOmegaExt~{i}, dGammaExt}]];
          SolveAgain[DDM_2~{i}];
        EndFor
        For ii In {0:#myD()-1}
          i = myD(ii);
          For jj In {0:#myD~{i}()-1}
            j = myD~{i}(jj);
            GenerateRHSGroup[G~{j}~{i},Region[{Sigma~{i}~{j}}]];
            SolveAgain[G~{j}~{i}];
          EndFor
          If(ABC_ORDER != 0)
            For kk In {0:#ListGammaExt~{i}()-1}
              k=ListGammaExt~{i}(kk);
              For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
                j=ListdGammaExt~{i}~{k}(index);
                l=ListdGammaExt~{i}~{k}(index+1);
                GenerateRHSGroup[H~{j}~{i}~{l}~{k}, Region[{Apt~{j}~{i}~{l}~{k}}]];
                SolveAgain[H~{j}~{i}~{l}~{k}];
              EndFor
            EndFor
          EndIf
        EndFor
      
        For ii In {0:#myD()-1}
          i = myD(ii);
          For jj In {0:#myD~{i}()-1}
            j = myD~{i}(jj);
            PostOperation[G~{j}~{i}];
          EndFor
          If(ABC_ORDER != 0)
            For kk In {0:#ListGammaExt~{i}()-1}
              k=ListGammaExt~{i}(kk);
              For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
                j=ListdGammaExt~{i}~{k}(index);
                l=ListdGammaExt~{i}~{k}(index+1);
                PostOperation[H~{j}~{i}~{l}~{k}];
              EndFor
            EndFor
          EndIf
        EndFor
      }
      // Compute solution
      Evaluate[$PhysicalSource = 1];
      Evaluate[$TCSource = 1];

      For ii In {0:#myD()-1}
        i = myD(ii);
        UpdateConstraint[DDM_2~{i}, dOmegaInt~{i}, Assign];
        GenerateRHSGroup[DDM_2~{i}, Region[{Sigma~{i}, TrdOmegaInt~{i}, dOmegaExt~{i}, dGammaExt}]];
        SolveAgain[DDM_2~{i}];
      EndFor
      PostOperation[DDM];
    }
  }
}

// Post Processing and Post Operation specific to DDM-2 algorithm
// ================
PostProcessing{
  For ii In {0:#myD()-1}
    i = myD(ii);
    For kk In {0:#ListGammaExt~{i}()-1}
      k=ListGammaExt~{i}(kk);
      For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
        j=ListdGammaExt~{i}~{k}(index);
        l=ListdGammaExt~{i}~{k}(index+1);
        {Name H~{j}~{i}~{l}~{k}; NameOfFormulation H~{j}~{i}~{l}~{k};
          Quantity {
           {Name h~{j}~{i}~{l}~{k}; Value {Local { [{h~{j}~{i}~{l}~{k}}] ; In Apt~{j}~{i}~{l}~{k}; Jacobian JLin; }}}
          }
        }
      EndFor
    EndFor
  EndFor
}

PostOperation{
  For ii In {0:#myD()-1}
    i = myD(ii);
    For kk In {0:#ListGammaExt~{i}()-1}
      k=ListGammaExt~{i}(kk);
      For index In {0:#ListdGammaExt~{i}~{k}()-1:2}
        j=ListdGammaExt~{i}~{k}(index);
        l=ListdGammaExt~{i}~{k}(index+1);
        {Name H~{j}~{i}~{l}~{k}; NameOfPostProcessing H~{j}~{i}~{l}~{k};
          Operation {Print [h~{j}~{i}~{l}~{k}, OnElementsOf Apt~{j}~{i}~{l}~{k}, StoreInField tag_h~{j}~{i}~{l}~{k}];}
        }
      EndFor
    EndFor
  EndFor
}
